#!/usr/bin/env perl

# This script automates the process of generating a set of evolved
# sequences using specific evolution parameters for seq-gen and
# sgEvolver

#
# This script reads configuration parameters from a simujobparams.pm file
# The simujobparams.pm file must be manually edited and contains additional
# documentation.
#
# (c) 2004, 2005 Aaron Darling, All rights reserved
#

use strict;
use POSIX;
require simujobparams;

# do the work:
# generate job descriptions that can be used to 
# perform simulation, alignment, and scoring

my $total_runs = 0;
my $x_max = $simujobparams::x_start + $simujobparams::x_increment * $simujobparams::x_count;
my $y_max = $simujobparams::y_start + $simujobparams::y_increment * $simujobparams::y_count;

# perform a mutation rate hack for nucleotide substition:
# seq-gen doesn't like a 0 mutation rate so just set the
# rate to be abysmally small.
my $nt_sub_hack_rate = 0.000000000000001; # 10e-13 is less than 1 mutation in an entire human genome
if( $simujobparams::x_variable eq "nt_substitution" && $simujobparams::x_start <= 0 ){
	$simujobparams::x_start = $nt_sub_hack_rate;
}
if( $simujobparams::y_variable eq "nt_substitution" && $simujobparams::y_start <= 0 ){
	$simujobparams::y_start = $nt_sub_hack_rate;
}

my $x_valI = 0;
my $y_valI = 0;
for( my $repI = 0; $repI < $simujobparams::repetitions; $repI++ ){
	$x_valI = 0;
	for( my $x_val = $simujobparams::x_start; $x_val < $x_max; $x_val += $simujobparams::x_increment ){
# modulate nt substitution rate on x-axis
		$x_valI++;
		$y_valI = 0;
		$simujobparams::nt_sub_scale = $x_val if $simujobparams::x_variable eq "nt_substitution";
		$simujobparams::indel_rate = $x_val if $simujobparams::x_variable eq "indels";
		$simujobparams::small_ht_rate = $x_val if $simujobparams::x_variable eq "small_ht";
		$simujobparams::large_ht_rate = $x_val if $simujobparams::x_variable eq "large_ht";
		$simujobparams::inv_rate = $x_val if $simujobparams::x_variable eq "inversions";

		for( my $y_val = $simujobparams::y_start; $y_val < $y_max; $y_val += $simujobparams::y_increment ){
			$y_valI++;
	                
			$simujobparams::nt_sub_scale = $y_val if $simujobparams::y_variable eq "nt_substitution";
	                $simujobparams::indel_rate = $y_val if $simujobparams::y_variable eq "indels";
                	$simujobparams::small_ht_rate = $y_val if $simujobparams::y_variable eq "small_ht";
        	        $simujobparams::large_ht_rate = $y_val if $simujobparams::y_variable eq "large_ht";
	                $simujobparams::inv_rate = $y_val if $simujobparams::y_variable eq "inversions";

			if( $total_runs < $simujobparams::skip_runs ){
				$total_runs++;
				next;
			}
			next if( $simujobparams::last_run >= 0 && $total_runs > $simujobparams::last_run );

		# get some random seed values
			my $seqgen_random = int(rand(0x7FFFFFFF));
			my $seqgen_donor_random = int(rand(0x7FFFFFFF));
			my $sgEvolver_random = int(rand(0x7FFFFFFF));

		# make a directory for the job files
			my $job_dir = "alignjob.$total_runs";
			mkdir( $job_dir );
			chdir( $job_dir );
			$total_runs++;

		# stat the ancestral donor to get its file size
		# select random subsequences for ancestral and donor
			my @stat_info = stat($simujobparams::ancestral_donor);
			my $size_modulus = $stat_info[7] - 2 * $simujobparams::sequence_length;
			my $ancestral_start = POSIX::floor( rand( $size_modulus ) );
			my $donor_modulus = $stat_info[7] - $ancestral_start - 2 * $simujobparams::sequence_length;
			my $donor_start = $ancestral_start + $simujobparams::sequence_length + POSIX::floor( rand( $donor_modulus ) );

		# write alignment params to a file
			open( PARAMS, ">simujobparams.pm"  );
			print PARAMS "package simujobparams;\n\n";
			print PARAMS "\$seq_length=$simujobparams::sequence_length;\n";
			print PARAMS "\$nt_sub_scale=$simujobparams::nt_sub_scale;\n";
			print PARAMS "\$gamma_shape=$simujobparams::gamma_shape;\n";
			print PARAMS "\$indel_rate=$simujobparams::indel_rate;\n";
			print PARAMS "\$small_ht_rate=$simujobparams::small_ht_rate;\n";
			print PARAMS "\$small_ht_size=$simujobparams::small_ht_size;\n";
			print PARAMS "\$large_ht_rate=$simujobparams::large_ht_rate;\n";
			print PARAMS "\$large_ht_min=$simujobparams::large_ht_min;\n";
			print PARAMS "\$large_ht_max=$simujobparams::large_ht_max;\n";
			print PARAMS "\$inv_rate=$simujobparams::inv_rate;\n";
			print PARAMS "\$inv_size=$simujobparams::inv_size;\n";
			print PARAMS "\$nt_a_freq=$simujobparams::nt_a_freq;\n";
			print PARAMS "\$nt_c_freq=$simujobparams::nt_c_freq;\n";
			print PARAMS "\$nt_g_freq=$simujobparams::nt_g_freq;\n";
			print PARAMS "\$nt_t_freq=$simujobparams::nt_t_freq;\n";

			print PARAMS "\$tree_filename=\"$simujobparams::tree_filename\";\n";
			print PARAMS "\$ancestral_donor=\"$simujobparams::ancestral_donor\";\n";
			print PARAMS "\$ancestral_seq_name=\"$simujobparams::ancestral_seq_name\";\n";
			print PARAMS "\$donor_seq_name=\"$simujobparams::donor_seq_name\";\n";
			print PARAMS "\$seqgen_out_name=\"$simujobparams::seqgen_out_name\";\n";
			print PARAMS "\$evolved_seqs_name=\"$simujobparams::evolved_seqs_name\";\n";
			print PARAMS "\$evolved_seqs_fname=\"$simujobparams::evolved_seqs_fname\";\n";

			print PARAMS "\$ancestral_start=$ancestral_start;\n";
			print PARAMS "\$donor_start=$donor_start;\n";

			print PARAMS "\$seqgen_random=$seqgen_random;\n";
			print PARAMS "\$seqgen_donor_random=$seqgen_donor_random;\n";
			print PARAMS "\$sgEvolver_random=$sgEvolver_random;\n";

			close PARAMS;
		
		# generate a tree file
			open( TREEOUT, ">$simujobparams::tree_filename" );
			print TREEOUT $simujobparams::phylogeny;
			close TREEOUT;
			print "Run: $total_runs\n";
		# leave the directory
			chdir( ".." );
                # kludge for seq-gen's inability to accept a 0 mutation rate
	                if( $simujobparams::y_variable eq "nt_substitution" && $y_val == $nt_sub_hack_rate ){
        	                $y_val = 0;
                	}
		}
		if( $simujobparams::x_variable eq "nt_substitution" && $x_val == $nt_sub_hack_rate ){
			$x_val = 0;
		}
	}
}

my $condor_job_file = "mauveAlign.condor";
open( JOB_FILE, ">$condor_job_file" );
print JOB_FILE qq{
####################                   
# Condor job submission for mauveAligner
####################                                                   

universe       = vanilla
Executable     = simujobrun.pl
arguments      = mauve    # should be one of mauve, mlagan, slagan, or mavid
Requirements   = Memory >= 128 && OpSys == "LINUX" && Arch =="INTEL"
Rank = Memory >= 256
Image_Size     = 900 Meg                                                

should_transfer_files = YES
when_to_transfer_output = ON_EXIT
transfer_input_files = $simujobparams::tree_filename, $simujobparams::evolution_params_file
notification = Never
coresize = 1000
copy_to_spool = False

Error   = simujobrun.err
Output  = simujobrun.out
initialdir = alignjob.\$(Process)
Log     = simujobrun.log

Queue $total_runs
};

close JOB_FILE;

my $run_description_file = "run_description.txt";
open( RUN_FILE, ">$run_description_file" );
# print out the number of cols, rows, and the increments used for each
print RUN_FILE "$x_valI\t$y_valI\t$simujobparams::x_increment\t$simujobparams::y_increment\n";
close RUN_FILE;


